<?xml version='1.0'?> 
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
                xmlns:fo="http://www.w3.org/1999/XSL/Format" version="1.1">

  <xsl:import href="/share/XML/docbook-xsl-1.60.1/fo/docbook.xsl"/> 
  <xsl:include href="ProgDocBook.xsl" />
  <xsl:param name="paper.type" select="'A4'"/>
  <xsl:param name="draft.watermark.image" select="''"/>
  <xsl:param name="generate.toc" select="nop"/>
  <xsl:param name="header.rule" select="0"/>
  <xsl:param name="footer.rule" select="0"/>

<!-- To avoid page numbers -->
<xsl:template name="footer.content">
</xsl:template>

<!-- To avoid page hreadings -->
<xsl:template name="header.content">
</xsl:template>

<!-- Following are the templates which handle the ProgDocBook elements.
     At the end of this file there are some templates which override templates
     from the standard "DocBook XSL Stylesheets" in order to make the 
     "listing" Element available, which was introduced by ProgDocBook.
-->

<xsl:strip-space elements="*"/>

<xsl:param name="indent-inc" select="'  '"/>

<xsl:template match="SourceBase"/>

<xsl:template match="doc" mode="PrettyPrint"/>

<xsl:template match="doc-orig" mode="PrettyPrint"/>

<xsl:template name="javadoc" mode="PrettyPrint">
  <xsl:param name="indent"/>
  <xsl:value-of select="concat($indent, '/**&#xA;')"/>
  <xsl:for-each select="doc-orig/text">
    <xsl:value-of select="' *'"/>
    <emphasis role="italic">
      <xsl:value-of select="concat(substring-after(@value, '*'), '&#xA;')"/>
    </emphasis>
  </xsl:for-each>
  <xsl:value-of select="concat($indent, ' */&#xA;')"/>
</xsl:template>

<xsl:template match="comment()" mode="PrettyPrint">
  <xsl:if test="contains(string(.), 'empty-line')">
    <xsl:text>&#xA;</xsl:text>
  </xsl:if>
</xsl:template>

<xsl:template match="Referenced" mode="PrettyPrint">
  <xsl:variable name="anchor" select="@ID"/>
  <xsl:if test="//listing[@id=$anchor]">
    <xsl:text> (Referenced in </xsl:text>
    <xsl:element name="xref">
      <xsl:attribute name="linkend">
        <xsl:value-of select="@ID"/>
      </xsl:attribute>
    </xsl:element>
    <xsl:text>)</xsl:text>
  </xsl:if>
</xsl:template>

<xsl:template match="Include" mode="PrettyPrint">
  <xsl:param name="indent"/>
  <xsl:choose>
    <xsl:when test="ancestor::listing/attribute::recursive-include='true'">
      <xsl:apply-templates select="./*" mode="PrettyPrint">
        <xsl:with-param name="indent" select="$indent"/>
      </xsl:apply-templates>
    </xsl:when>
    <xsl:otherwise>
      <xsl:value-of select="$indent"/>
      <emphasis role="bold-italic">
        <xsl:text>&lt;</xsl:text>
        <xsl:if test="@label">
          <xsl:value-of select="@label"/>
        </xsl:if>
        <xsl:variable name="anchor" select="@ID"/>
        <xsl:if test="//listing[@id=$anchor]">
          <xsl:text> (see </xsl:text>
          <xsl:element name="xref">
            <xsl:attribute name="linkend">
              <xsl:value-of select="@ID"/>
            </xsl:attribute>
          </xsl:element>
          <xsl:text>)</xsl:text>
        </xsl:if>
        <xsl:text>&gt;&#xA;</xsl:text>
      </emphasis>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

<xsl:template match="new-array" mode="PrettyPrint">
  <emphasis role="bold">
    <xsl:value-of select="concat('new ', @type)"/>
  </emphasis>
  <xsl:if test="count(./*) > 0">
    <xsl:text> { </xsl:text>
    <xsl:for-each select="*">
      <xsl:if test="position()!=1">
        <xsl:value-of select="', '"/>
      </xsl:if>
      <xsl:apply-templates select="." mode="PrettyPrint"/>
    </xsl:for-each>
    <xsl:text> }</xsl:text>
  </xsl:if>
</xsl:template>

<xsl:template match="var-def" mode="PrettyPrint">
  <xsl:param name="indent"/>
  <xsl:value-of select="$indent"/>
  <emphasis role="bold"><xsl:value-of select="@type"/></emphasis>
  <xsl:text> </xsl:text>
  <xsl:value-of select="@name"/>
  <xsl:if test="count(./*) = 1">
    <xsl:text> = </xsl:text>
    <xsl:apply-templates select="./*[1]" mode="PrettyPrint"/>
  </xsl:if>
  <xsl:if test="not(parent::params|parent::init)">
    <xsl:text>;&#xA;</xsl:text>
  </xsl:if>
</xsl:template>

<xsl:template match="int" mode="PrettyPrint">
  <xsl:param name="indent"/>
  <xsl:value-of select="@value"/>
</xsl:template>

<xsl:template match="String" mode="PrettyPrint">
  <xsl:param name="indent"/>
  <xsl:value-of select="concat(concat('&#x22;',@value), '&#x22;')"/>
</xsl:template>

<xsl:template match="comment" mode="PrettyPrint">
  <xsl:param name="indent"/>
  <xsl:value-of select="concat($indent, '//')"/>
  <emphasis role="italic"><xsl:value-of select="@value"/></emphasis>
  <xsl:text>&#xA;</xsl:text>
</xsl:template>

<xsl:template match="for-loop" mode="PrettyPrint">
  <xsl:param name="indent"/>
  <xsl:value-of select="$indent"/>
  <emphasis role="bold">for</emphasis>
  <xsl:text>(</xsl:text>
  <xsl:apply-templates select="init/*" mode="PrettyPrint"/>
  <xsl:text>;</xsl:text>
  <xsl:apply-templates select="condition" mode="PrettyPrint"/>
  <xsl:text>;</xsl:text>
  <xsl:apply-templates select="update" mode="PrettyPrint"/>
  <xsl:text>) </xsl:text>
  <xsl:if test="./body[@single='false']">
    <xsl:text>{&#xA;</xsl:text>
    <xsl:apply-templates select="body" mode="PrettyPrint">
      <xsl:with-param name="indent" select="concat($indent,$indent-inc)"/>
    </xsl:apply-templates>
    <xsl:value-of select="$indent"/>
    <xsl:text>}&#xA;</xsl:text>
  </xsl:if>
  <xsl:if test="./body[@single='true']">
    <xsl:apply-templates select="body" mode="PrettyPrint">
      <xsl:with-param name="indent" select="''"/>
    </xsl:apply-templates>
  </xsl:if>
</xsl:template>

<xsl:template match="while-loop" mode="PrettyPrint">
  <xsl:param name="indent"/>
  <xsl:variable name="new-indent" select="concat($indent,$indent-inc)"/>
  <xsl:value-of select="$indent"/>
  <emphasis role="bold">while</emphasis>
  <xsl:text>(</xsl:text>
  <xsl:apply-templates select="condition" mode="PrettyPrint">
  </xsl:apply-templates>
  <xsl:text>) </xsl:text>
  <xsl:if test="./body[@single='false']">
    <xsl:text>{&#xA;</xsl:text>
    <xsl:apply-templates select="body" mode="PrettyPrint">
      <xsl:with-param name="indent" select="$new-indent"/>
    </xsl:apply-templates>
    <xsl:value-of select="$indent"/>
    <xsl:text>}&#xA;</xsl:text>
  </xsl:if>
  <xsl:if test="./body[@single='true']">
    <xsl:apply-templates select="body" mode="PrettyPrint">
      <xsl:with-param name="indent" select="''"/>
    </xsl:apply-templates>
  </xsl:if>
</xsl:template>

<xsl:template match="class" mode="PrettyPrint">
  <xsl:param name="indent"/>
  <xsl:variable name="new-indent" select="concat($indent,$indent-inc)"/>
  <xsl:if test="ancestor::listing/attribute::java-doc='true' and doc-orig">
    <xsl:call-template name="javadoc">
      <xsl:with-param name="indent" select="$indent"/>
    </xsl:call-template>
  </xsl:if>
  <xsl:value-of select="$indent"/>
  <emphasis role="bold">
  <xsl:value-of select="concat(@visibility, ' class ')"/>
  </emphasis>
  <xsl:value-of select="concat(@name, '')"/>
  <xsl:value-of select="' {&#xA;'"/>
  <xsl:apply-templates select="*|comment()" mode="PrettyPrint">
    <xsl:with-param name="indent" select="$new-indent"/>
  </xsl:apply-templates>
  <xsl:value-of select="$indent"/>
  <xsl:text>}&#xA;</xsl:text>
</xsl:template>

<xsl:template match="method" mode="PrettyPrint">
  <xsl:param name="indent"/>
  <xsl:variable name="new-indent" select="concat($indent,$indent-inc)"/>
  <xsl:if test="ancestor::listing/attribute::java-doc='true' and doc-orig">
    <xsl:call-template name="javadoc">
      <xsl:with-param name="indent" select="$indent"/>
    </xsl:call-template>
  </xsl:if>
  <xsl:value-of select="$indent"/>
  <emphasis role="bold">
  <xsl:value-of select="concat(@visibility, ' ')"/>
  <xsl:value-of select="concat(@retType, ' ')"/>
  </emphasis>
  <xsl:value-of select="concat(@name, '(')"/>
  <xsl:apply-templates select="params" mode="PrettyPrint">
    <xsl:with-param name="indent" select="$indent"/>
  </xsl:apply-templates> 
  <xsl:value-of select="') {&#xA;'"/>
  <xsl:apply-templates select="*[name()!='params']|comment()" mode="PrettyPrint">
    <xsl:with-param name="indent" select="$new-indent"/>
  </xsl:apply-templates>
  <xsl:value-of select="$indent"/>
  <xsl:text>}&#xA;</xsl:text>
</xsl:template>

<xsl:template match="constructor" mode="PrettyPrint">
  <xsl:param name="indent"/>
  <xsl:variable name="new-indent" select="concat($indent,$indent-inc)"/>
  <!-- We don't print the default constructor generated by the compiler! -->
  <xsl:if test="count(.)!=1">
  <xsl:value-of select="$indent"/>
  <emphasis role="bold">
  <xsl:value-of select="@visibility"/>
  <xsl:value-of select="concat(@retType, ' ')"/>
  </emphasis>
  <xsl:value-of select="concat(@name, '(')"/>
  <xsl:apply-templates select="params" mode="PrettyPrint">
    <xsl:with-param name="indent" select="$indent"/>
  </xsl:apply-templates> 
  <xsl:value-of select="') {&#xA;'"/>
  <xsl:apply-templates select="*[name()!='params']|comment()" mode="PrettyPrint">
    <xsl:with-param name="indent" select="$new-indent"/>
  </xsl:apply-templates>
  <xsl:value-of select="$indent"/>
  <xsl:text>}&#xA;</xsl:text>
  </xsl:if>
</xsl:template>

<xsl:template match="if" mode="PrettyPrint">
  <xsl:param name="indent"/>
  <xsl:variable name="new-indent" select="concat($indent,$indent-inc)"/>
  <xsl:value-of select="$indent"/>
  <emphasis role="bold">if</emphasis>
  <xsl:text>(</xsl:text>
  <xsl:apply-templates select="condition" mode="PrettyPrint">
  </xsl:apply-templates>
  <xsl:text>) </xsl:text>
  <xsl:if test="./then[@single='false']">
    <xsl:text>{&#xA;</xsl:text>
    <xsl:apply-templates select="then" mode="PrettyPrint">
      <xsl:with-param name="indent" select="$new-indent"/>
    </xsl:apply-templates>
    <xsl:value-of select="$indent"/>
    <xsl:text>}&#xA;</xsl:text>
  </xsl:if>
  <xsl:if test="./then[@single='true']">
    <xsl:apply-templates select="then" mode="PrettyPrint">
      <xsl:with-param name="indent" select="''"/>
    </xsl:apply-templates>
  </xsl:if>
  <xsl:if test="else">
    <xsl:value-of select="$indent"/>
    <xsl:text>else </xsl:text>
    <xsl:if test="./then[@single='false']">
      <xsl:text>{&#xA;</xsl:text>
      <xsl:apply-templates select="then" mode="PrettyPrint">
        <xsl:with-param name="indent" select="$new-indent"/>
      </xsl:apply-templates>
      <xsl:value-of select="$indent"/>
      <xsl:text>}&#xA;</xsl:text>
    </xsl:if>
    <xsl:if test="./then[@single='true']">
      <xsl:apply-templates select="then" mode="PrettyPrint">
        <xsl:with-param name="indent" select="''"/>
      </xsl:apply-templates>
    </xsl:if>
  </xsl:if>
</xsl:template>

<xsl:template match="condition" mode="PrettyPrint">
  <xsl:param name="indent"/>
  <xsl:apply-templates mode="PrettyPrint"> 
    <xsl:with-param name="indent" select="$indent"/>
  </xsl:apply-templates> 
</xsl:template>

<xsl:template match="update" mode="PrettyPrint">
  <xsl:param name="indent"/>
  <xsl:apply-templates mode="PrettyPrint"> 
    <xsl:with-param name="indent" select="$indent"/>
  </xsl:apply-templates> 
</xsl:template>

<xsl:template match="body" mode="PrettyPrint">
  <xsl:param name="indent"/>
  <xsl:apply-templates mode="PrettyPrint"> 
    <xsl:with-param name="indent" select="$indent"/>
  </xsl:apply-templates> 
</xsl:template>

<xsl:template match="then" mode="PrettyPrint">
  <xsl:param name="indent"/>
  <xsl:apply-templates mode="PrettyPrint"> 
    <xsl:with-param name="indent" select="$indent"/>
  </xsl:apply-templates> 
</xsl:template>

<xsl:template match="else" mode="PrettyPrint">
  <xsl:param name="indent"/>
  <xsl:apply-templates mode="PrettyPrint"> 
    <xsl:with-param name="indent" select="$indent"/>
  </xsl:apply-templates> 
</xsl:template>

<xsl:template match="binary-expression" mode="PrettyPrint">
  <xsl:param name="indent"/>
  <xsl:apply-templates select="./*[1]" mode="PrettyPrint">
    <xsl:with-param name="indent" select="$indent"/>
  </xsl:apply-templates> 
  <xsl:value-of select="concat(' ', concat(@operator, ' '))"/>
  <xsl:apply-templates select="./*[2]" mode="PrettyPrint"> 
    <xsl:with-param name="indent" select="$indent"/>
  </xsl:apply-templates> 
</xsl:template>

<xsl:template match="select" mode="PrettyPrint">
  <xsl:param name="indent"/>
  <xsl:apply-templates select="./*" mode="PrettyPrint">
    <xsl:with-param name="indent" select="$indent"/>
  </xsl:apply-templates> 
  <xsl:value-of select="concat('&#x2E;', @name)"/>
</xsl:template>

<xsl:template match="type" mode="PrettyPrint">
  <xsl:param name="indent"/>
  <emphasis role="bold">
    <xsl:value-of select="@name"/>
  </emphasis>
</xsl:template>

<xsl:template match="assign" mode="PrettyPrint">
  <xsl:param name="indent"/>
  <xsl:value-of select="$indent"/>
  <xsl:apply-templates select="./*[1]" mode="PrettyPrint">
    <xsl:with-param name="indent" select="$indent"/>
  </xsl:apply-templates> 
  <xsl:value-of select="' = '"/>
  <xsl:apply-templates select="./*[2]" mode="PrettyPrint"> 
    <xsl:with-param name="indent" select="$indent"/>
  </xsl:apply-templates> 
  <xsl:value-of select="';&#xA;'"/>
</xsl:template>

<xsl:template match="indexed" mode="PrettyPrint">
  <xsl:param name="indent"/>
  <xsl:apply-templates select="./*[1]" mode="PrettyPrint">
    <xsl:with-param name="indent" select="$indent"/>
  </xsl:apply-templates> 
  <xsl:value-of select="'['"/>
  <xsl:apply-templates select="./*[2]" mode="PrettyPrint"> 
    <xsl:with-param name="indent" select="$indent"/>
  </xsl:apply-templates> 
  <xsl:value-of select="']'"/>
</xsl:template>

<xsl:template match="unary-expression" mode="PrettyPrint">
  <xsl:param name="indent"/>
  <xsl:if test="parent::body|parent::then|parent::else">
    <xsl:value-of select="$indent"/>
  </xsl:if>
  <xsl:if test="@post='false'">
    <xsl:value-of select="@operator"/>
  </xsl:if>
  <xsl:apply-templates select="./*[1]" mode="PrettyPrint">
    <xsl:with-param name="indent" select="$indent"/>
  </xsl:apply-templates> 
  <xsl:if test="@post='true'">
    <xsl:value-of select="@operator"/>
  </xsl:if>
  <xsl:if test="parent::body|parent::then|parent::else">
    <xsl:value-of select="';&#xA;'"/>
  </xsl:if>
</xsl:template>

<xsl:template match="apply" mode="PrettyPrint">
  <xsl:param name="indent"/>
  <xsl:if test="parent::body|parent::then|parent::else|parent::method">
    <xsl:value-of select="$indent"/>
  </xsl:if>
  <xsl:if test="count(./*[name()!='params']) > 0">
    <xsl:apply-templates select="./*[1]" mode="PrettyPrint">
      <xsl:with-param name="indent" select="$indent"/>
    </xsl:apply-templates>
    <xsl:text>.</xsl:text>
  </xsl:if>
  <xsl:value-of select="concat(@name, '(')"/>
  <xsl:apply-templates select="params" mode="PrettyPrint">
    <xsl:with-param name="indent" select="$indent"/>
  </xsl:apply-templates> 
  <xsl:value-of select="')'"/>
  <xsl:if test="parent::body|parent::then|parent::else|parent::method">
    <xsl:value-of select="';&#xA;'"/>
  </xsl:if>
</xsl:template>

<xsl:template match="params" mode="PrettyPrint">
  <xsl:for-each select="*">
    <xsl:if test="position()!=1">
      <xsl:value-of select="', '"/>
    </xsl:if>
    <xsl:apply-templates select="." mode="PrettyPrint"/>
  </xsl:for-each>
</xsl:template>

<xsl:template match="var" mode="PrettyPrint">
  <xsl:value-of select="@name"/>
</xsl:template>

<xsl:template match="paren" mode="PrettyPrint">
  <xsl:param name="indent"/>
  <xsl:value-of select="'('"/>
  <xsl:apply-templates select="./*[1]" mode="PrettyPrint">
    <xsl:with-param name="indent" select="$indent"/>
  </xsl:apply-templates> 
  <xsl:value-of select="')'"/>
</xsl:template>

<xsl:template match="Listing" mode="IncludeMode">
  <xsl:variable name="sourceBase" select="preceding::SourceBase"/>
  <xsl:variable name="anchor" select="@anchor"/>
  <xsl:variable name="java-doc" select="@java-doc"/>
  <xsl:variable name="recursive-include" select="@recursive-include"/>
  <xsl:choose>
    <xsl:when test="@type='include'">
      <xsl:variable name="code" 
                    select="document(concat(substring-before(@href, '.'), '.xml'), $sourceBase)//Include[@ID=$anchor]/*"/>
      <xsl:call-template name="ListingA" mode="IncludeMode">
        <xsl:with-param name="code" select="$code"/>
        <xsl:with-param name="anchor" select="$anchor"/>
        <xsl:with-param name="sourceBase" select="$sourceBase"/>
        <xsl:with-param name="java-doc" select="$java-doc"/>
        <xsl:with-param name="recursive-include" select="$recursive-include"/>
      </xsl:call-template>
    </xsl:when>
    <xsl:when test="@type='select'">
      <xsl:variable name="kind" select="@kind"/>
      <xsl:variable name="name" select="@name"/>
      <xsl:variable name="code" 
                    select="document(concat(substring-before(@href, '.'), '.xml'), $sourceBase)//*[name()=$kind and @name=$name]"/>
      <xsl:call-template name="ListingA" mode="IncludeMode">
        <xsl:with-param name="type" select="select"/>
        <xsl:with-param name="kind" select="$kind"/>
        <xsl:with-param name="code" select="$code"/>
        <xsl:with-param name="anchor" select="$anchor"/>
        <xsl:with-param name="sourceBase" select="$sourceBase"/>
        <xsl:with-param name="java-doc" select="$java-doc"/>
        <xsl:with-param name="recursive-include" select="$recursive-include"/>
      </xsl:call-template>
    </xsl:when>
    <xsl:when test="@type='api'">
      <xsl:variable name="kind" select="@kind"/>
      <xsl:variable name="name" select="@name"/>
      <xsl:variable name="code" 
                    select="document(concat(substring-before(@href, '.'), '.xml'), $sourceBase)//*[name()=$kind and @name=$name]"/>
      <xsl:call-template name="ListingB" mode="IncludeMode">
        <xsl:with-param name="type" select="select"/>
        <xsl:with-param name="kind" select="$kind"/>
        <xsl:with-param name="code" select="$code"/>
        <xsl:with-param name="anchor" select="$anchor"/>
        <xsl:with-param name="name" select="$name"/>
        <xsl:with-param name="sourceBase" select="$sourceBase"/>
      </xsl:call-template>
    </xsl:when>
  </xsl:choose>
</xsl:template>

<xsl:template name="ListingA" mode="IncludeMode">
  <xsl:param name="type" default="include"/>
  <xsl:param name="kind"/>
  <xsl:param name="code"/>
  <xsl:param name="anchor"/>
  <xsl:param name="sourceBase"/>
  <xsl:param name="java-doc"/>
  <xsl:param name="recursive-include"/>
  <xsl:variable name="listing">
    <xsl:element name="listing">
      <xsl:attribute name="id">
        <xsl:value-of select="$anchor"/>
      </xsl:attribute>
      <xsl:attribute name="java-doc">
        <xsl:value-of select="$java-doc"/>
      </xsl:attribute>
      <xsl:attribute name="recursive-include">
        <xsl:value-of select="$recursive-include"/>
      </xsl:attribute>
      <xsl:element name="title">
        <xsl:value-of select="@href"/>
        <xsl:text> [Lines  </xsl:text>
        <xsl:for-each select="$code">
	  <xsl:if test="position()=1">
	    <xsl:value-of select="@line"/>
            <xsl:text> to </xsl:text>
	  </xsl:if>
	</xsl:for-each>
        <xsl:if test="@type='include'">
          <xsl:for-each select="$code/parent::*//comment()[contains(string(),'line=')]">
  	    <xsl:if test="position()=last()">
	      <xsl:value-of select="substring-before(substring-after(string(), '&#x22;'), '&#x22;') - 1"/>
              <xsl:text>]</xsl:text>
	    </xsl:if>
	  </xsl:for-each>
        </xsl:if>
        <xsl:if test="@type='select'">
          <xsl:for-each select="$code//*">
  	    <xsl:if test="position()=last()">
  	      <xsl:if test="$kind='method'">
	        <xsl:value-of select="@line + 1"/>
	      </xsl:if>
  	      <xsl:if test="$kind='class'">
	        <xsl:value-of select="@line + 2"/>
	      </xsl:if>
  	      <xsl:if test="not($kind='class' or $kind='method')">
	        <xsl:value-of select="@line"/>
	      </xsl:if>
              <xsl:text>]</xsl:text>
	    </xsl:if>
	  </xsl:for-each>
        </xsl:if>
	<xsl:for-each select="document(concat(substring-before(@href, '.'), '.xml'),$sourceBase)//Include[@ID=$anchor]/ancestor::Include">
<!-- We can't decide at this stage if the parent listing of a nested listing-->
<!-- is also included in this document. So insted of creating a "Referenced -->
<!-- by.." message we create a "Referenced" element which will be processed -->
<!-- in the pretty-printing step.                                           -->
          <xsl:element name="Referenced">
            <xsl:attribute name="ID">
              <xsl:value-of select="@ID"/>
            </xsl:attribute>
          </xsl:element>
	</xsl:for-each>
      </xsl:element>
      <xsl:element name="synopsis">
        <xsl:copy-of select="$code"/>
      </xsl:element>
    </xsl:element>
  </xsl:variable>
  <xsl:copy-of select="$listing"/>
</xsl:template>

<xsl:template name="ListingB" mode="IncludeMode">
  <xsl:param name="type" default="include"/>
  <xsl:param name="kind"/>
  <xsl:param name="code"/>
  <xsl:param name="anchor"/>
  <xsl:param name="name"/>
  <xsl:param name="sourceBase"/>
  <xsl:variable name="listing">
    <xsl:element name="listing">
      <xsl:attribute name="id"><xsl:value-of select="$anchor"/></xsl:attribute>
      <xsl:element name="title">
        <xsl:choose>
  	  <xsl:when test="$kind='method'">
            <xsl:value-of select="concat(concat('Method ', $name), ': ')"/>
            <xsl:for-each select="$code/doc/text[1]">
              <xsl:value-of select="substring-before(@value, '. ')"/>
	    </xsl:for-each>
	  </xsl:when>
  	  <xsl:when test="$kind='class'">
            <xsl:value-of select="concat('Class declaration for ', $name)"/>
	  </xsl:when>
        </xsl:choose>
      </xsl:element>
      <xsl:for-each select="$code/doc/text">
        <xsl:element name="para">
          <emphasis role="sans">
            <xsl:value-of select="@value"/>
          </emphasis>
        </xsl:element>
      </xsl:for-each>
      <xsl:element name="synopsis">
        <xsl:attribute name="Role">API</xsl:attribute>
        <xsl:for-each select="$code">
          <emphasis role="bold">
            <xsl:value-of select="concat(@visibility, ' ')"/>
            <xsl:value-of select="concat(@retType, ' &#xA;')"/>
            <xsl:value-of select="@name"/>
          </emphasis>
          <xsl:value-of select="'('"/>
	  <xsl:apply-templates select="params" mode="PrettyPrint"/>
	  <xsl:value-of select="');&#xA;'"/>
	</xsl:for-each>
      </xsl:element>
      <xsl:element name="para">

        <informaltable frame="none">
          <tgroup cols='2' align='left' colsep='0' rowsep='0'>
            <colspec colwidth="1*" colname='c1'/>
            <colspec colwidth="5*" colname='c2'/>
            <tbody>

         <xsl:for-each select="$code/doc/tag[@name='param']">
	  <xsl:if test="position()=1">

            <row rowsep="1">
              <entry namest="c1" nameend="c2"><emphasis role="sans-bold">
                <xsl:value-of select="'Parameters:'"/>
                </emphasis>
              </entry>
            </row>

	  </xsl:if>

          <row>
            <entry><emphasis role="mono">
              <xsl:value-of select="@paramName"/>
              </emphasis>
            </entry>
            <entry><emphasis role="sans">
              <xsl:value-of select="@value"/>
              </emphasis>
            </entry>
          </row>

	  <xsl:if test="position()=last()">
	  </xsl:if>
	</xsl:for-each>
        <xsl:for-each select="$code/doc/tag[@name='return']">
	  <xsl:if test="position()=1">

            <row rowsep="1">
              <entry namest="c1" nameend="c2"><emphasis role="sans-bold">
                <xsl:value-of select="'Return Value:'"/>
                </emphasis>
              </entry>
            </row>

	  </xsl:if>

          <row>
            <entry colname="c2"><emphasis role="sans">
              <xsl:value-of select="@value"/>
              </emphasis>
            </entry>
          </row>

	  <xsl:if test="position()=last()">
	  </xsl:if>
	</xsl:for-each>

            </tbody>
          </tgroup>
        </informaltable>

      </xsl:element>
    </xsl:element>
  </xsl:variable>
  <xsl:copy-of select="$listing"/>
</xsl:template>

<xsl:template match="/">
  <!-- First of all we do the inclusion stage -->
  <xsl:variable name="doc1">
    <xsl:copy>
      <xsl:apply-templates select="@*|node()|comment()" mode="IncludeMode"/>
    </xsl:copy>
  </xsl:variable>
  <!-- In a second step we pretty-print and link the included source code -->
  <xsl:variable name="doc2">
    <xsl:copy>
      <xsl:apply-templates select="$doc1" mode="PrettyPrint"/>
    </xsl:copy>
  </xsl:variable>
  <!-- Finally we hand over control to the original rule from 'docbook.xsl' -->
  <!-- which will be applied now to the new root element.                   -->
  <xsl:apply-templates select="$doc2" mode="originalRootRule"/>
</xsl:template>

<xsl:template match="*|@*|comment()" mode="IncludeMode">
  <xsl:copy>
    <xsl:apply-templates select="@*|node()|comment()" mode="IncludeMode"/>
  </xsl:copy>
</xsl:template>

<xsl:template match="*|@*" mode="PrettyPrint">
  <xsl:copy>
    <xsl:apply-templates select="@*|node()|comment()" mode="PrettyPrint"/>
  </xsl:copy>
</xsl:template>


<!-- Below this line overridden DocBook XSL templates -->


<!-- From 'params/shade.verbatim.style.xml' -->

<xsl:attribute-set condition="html" name="shade.verbatim.style">
  <xsl:attribute name="border">0</xsl:attribute>
  <xsl:attribute name="bgcolor">#FEFEFE</xsl:attribute>
</xsl:attribute-set>
<xsl:attribute-set condition="fo" name="shade.verbatim.style">
  <xsl:attribute name="background-color">#FEFEFE</xsl:attribute>
</xsl:attribute-set>

<!-- From 'params/table.cell.padding.xml' -->

<xsl:attribute-set name="table.cell.padding">
  <xsl:attribute name="padding-left">2pt</xsl:attribute>
  <xsl:attribute name="padding-right">2pt</xsl:attribute>
  <xsl:attribute name="padding-top">1pt</xsl:attribute>
  <xsl:attribute name="padding-bottom">1pt</xsl:attribute>
</xsl:attribute-set>

<!-- From 'params/formal.title.properties.xml' -->

<xsl:attribute-set name="formal.title.properties"
	           use-attribute-sets="normal.para.spacing">
  <xsl:attribute name="font-weight">bold</xsl:attribute>
  <xsl:attribute name="font-size">
    <xsl:value-of select="$body.font.master * 1"/>
    <xsl:text>pt</xsl:text>
  </xsl:attribute>
  <xsl:attribute name="hyphenate">false</xsl:attribute>
  <xsl:attribute name="space-after.minimum">0.4em</xsl:attribute>
  <xsl:attribute name="space-after.optimum">0.6em</xsl:attribute>
  <xsl:attribute name="space-after.maximum">0.8em</xsl:attribute>
</xsl:attribute-set>

<!-- From 'fo/param.xsl' -->

<xsl:attribute-set name="big-monospace.verbatim.properties"
                   use-attribute-sets="verbatim.properties">
  <xsl:attribute name="font-family">
    <xsl:value-of select="$monospace.font.family"/>
  </xsl:attribute>
  <xsl:attribute name="font-size">
    <xsl:value-of select="$body.font.master * 0.9"/>
    <xsl:text>pt</xsl:text>
  </xsl:attribute>
  <xsl:attribute name="border-color">#0000FF</xsl:attribute>
  <xsl:attribute name="border-style">solid</xsl:attribute>
  <xsl:attribute name="border-width">0.2pt</xsl:attribute>
  <xsl:attribute name="padding">4pt</xsl:attribute>
  <xsl:attribute name="start-indent">4pt</xsl:attribute>
  <xsl:attribute name="end-indent">4pt</xsl:attribute>
  <xsl:attribute name="background-color">#F8F8F8</xsl:attribute>
</xsl:attribute-set>

<xsl:attribute-set name="monospace.verbatim.properties"
                   use-attribute-sets="verbatim.properties">
  <xsl:attribute name="font-family">
    <xsl:value-of select="$monospace.font.family"/>
  </xsl:attribute>
  <xsl:attribute name="font-size">
    <xsl:value-of select="$body.font.master * 0.7"/>
    <xsl:text>pt</xsl:text>
  </xsl:attribute>
  <xsl:attribute name="border-color">#0000FF</xsl:attribute>
  <xsl:attribute name="border-style">solid</xsl:attribute>
  <xsl:attribute name="border-width">0.2pt</xsl:attribute>
  <xsl:attribute name="padding">4pt</xsl:attribute>
  <xsl:attribute name="start-indent">4pt</xsl:attribute>
  <xsl:attribute name="end-indent">4pt</xsl:attribute>
  <xsl:attribute name="background-color">#F8F8F8</xsl:attribute>
</xsl:attribute-set>

<!-- From 'fo/verbatim.xsl' -->

<xsl:template match="programlisting|screen|synopsis">
  <xsl:param name="suppress-numbers" select="'0'"/>
  <xsl:variable name="id"><xsl:call-template name="object.id"/></xsl:variable>

  <xsl:variable name="content">
    <xsl:choose>
      <xsl:when test="$suppress-numbers = '0'
                      and @linenumbering = 'numbered'
                      and $use.extensions != '0'
                      and $linenumbering.extension != '0'">
        <xsl:call-template name="number.rtf.lines">
          <xsl:with-param name="rtf">
            <xsl:apply-templates/>
          </xsl:with-param>
        </xsl:call-template>
      </xsl:when>
      <xsl:otherwise>
        <xsl:apply-templates/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:variable>

  <xsl:choose>
    <xsl:when test="$shade.verbatim != 0">
      <fo:block wrap-option='no-wrap'
                white-space-collapse='false'
                linefeed-treatment="preserve"
                xsl:use-attribute-sets="monospace.verbatim.properties shade.verbatim.style">

        <xsl:copy-of select="$content"/>
      </fo:block>
    </xsl:when>
    <xsl:when test="@Role = 'API'">
      <fo:block wrap-option='no-wrap'
                white-space-collapse='false'
                linefeed-treatment="preserve"
                xsl:use-attribute-sets="big-monospace.verbatim.properties">
        <xsl:copy-of select="$content"/>
      </fo:block>
    </xsl:when>
    <xsl:otherwise>
      <fo:block wrap-option='no-wrap'
                white-space-collapse='false'
                linefeed-treatment="preserve"
                xsl:use-attribute-sets="monospace.verbatim.properties">
        <xsl:copy-of select="$content"/>
      </fo:block>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

<!-- From 'fo/inline.xsl' -->

<xsl:template name="inline.bolditalicseq">
  <xsl:param name="content">
    <xsl:apply-templates/>
  </xsl:param>
  <fo:inline font-weight="bold" font-style="italic">
    <xsl:copy-of select="$content"/>
  </fo:inline>
</xsl:template>

<xsl:template name="inline.sansseq">
  <xsl:param name="content">
    <xsl:apply-templates/>
  </xsl:param>
  <fo:inline font-family="{$sans.font.family}" font-size="{$body.font.master*0.9}pt">
    <xsl:copy-of select="$content"/>
  </fo:inline>
</xsl:template>

<xsl:template name="inline.sansboldseq">
  <xsl:param name="content">
    <xsl:apply-templates/>
  </xsl:param>
  <fo:inline font-family="{$sans.font.family}" font-weight="bold" font-size="{$body.font.master*0.9}pt">
    <xsl:copy-of select="$content"/>
  </fo:inline>
</xsl:template>

<xsl:template match="emphasis">
  <xsl:variable name="depth">
    <xsl:call-template name="dot.count">
      <xsl:with-param name="string"><xsl:number level="multiple"/></xsl:with-param>
    </xsl:call-template>
  </xsl:variable>

  <xsl:choose>
    <xsl:when test="@role='bold'">
      <xsl:call-template name="inline.boldseq"/>
    </xsl:when>
    <xsl:when test="@role='underline'">
      <fo:inline text-decoration="underline">
        <xsl:call-template name="inline.charseq"/>
      </fo:inline>
    </xsl:when>
    <xsl:when test="@role='bold-italic'">
      <xsl:call-template name="inline.bolditalicseq"/>
    </xsl:when>
    <xsl:when test="@role='mono'">
      <xsl:call-template name="inline.monoseq"/>
    </xsl:when>
    <xsl:when test="@role='sans'">
      <xsl:call-template name="inline.sansseq"/>
    </xsl:when>
    <xsl:when test="@role='sans-bold'">
      <xsl:call-template name="inline.sansboldseq"/>
    </xsl:when>
    <xsl:when test="@role='strikethrough'">
      <fo:inline text-decoration="line-through">
        <xsl:call-template name="inline.charseq"/>
      </fo:inline>
    </xsl:when>
    <xsl:otherwise>
      <xsl:choose>
        <xsl:when test="$depth mod 2 = 1">
          <fo:inline font-style="normal">
            <xsl:apply-templates/>
          </fo:inline>
        </xsl:when>
        <xsl:otherwise>
          <xsl:call-template name="inline.italicseq"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

<!-- From 'fo/formal.xsl' -->

<xsl:template match="listing">
  <xsl:variable name="param.placement"
                select="substring-after(normalize-space($formal.title.placement),
                                        concat(local-name(.), ' '))"/>

  <xsl:variable name="placement">
    <xsl:choose>
      <xsl:when test="contains($param.placement, ' ')">
        <xsl:value-of select="substring-before($param.placement, ' ')"/>
      </xsl:when>
      <xsl:when test="$param.placement = ''">before</xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="$param.placement"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:variable>

  <!-- Get align value from internal mediaobject -->
  <xsl:variable name="align">
    <xsl:if test="mediaobject">
      <xsl:variable name="olist" select="mediaobject/imageobject
                     |mediaobject/imageobjectco
                     |mediaobject/videoobject
		     |mediaobject/audioobject
		     |mediaobject/textobject"/>

      <xsl:variable name="object.index">
        <xsl:call-template name="select.mediaobject.index">
          <xsl:with-param name="olist" select="$olist"/>
          <xsl:with-param name="count" select="1"/>
        </xsl:call-template>
      </xsl:variable>

      <xsl:variable name="object" select="$olist[position() = $object.index]"/>

      <xsl:value-of select="$object/imagedata[@align][1]/@align"/>
    </xsl:if>
  </xsl:variable>

  <xsl:choose>
    <xsl:when test="$align != ''">
      <fo:block>
	  <xsl:attribute name="text-align">
	    <xsl:value-of select="$align"/>
	  </xsl:attribute>
        <xsl:call-template name="formal.object">
          <xsl:with-param name="placement" select="$placement"/>
        </xsl:call-template>
      </fo:block>
    </xsl:when>
    <xsl:otherwise>
      <xsl:call-template name="formal.object">
        <xsl:with-param name="placement" select="$placement"/>
      </xsl:call-template>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

<xsl:template match="listing/title"></xsl:template>

<!-- From 'fo/xref.xsl' -->

<xsl:template match="figure|example|table|equation|listing" mode="xref-to">
  <xsl:param name="referrer"/>
  <xsl:param name="xrefstyle"/>

  <xsl:apply-templates select="." mode="object.xref.markup">
    <xsl:with-param name="purpose" select="'xref'"/>
    <xsl:with-param name="xrefstyle" select="$xrefstyle"/>
    <xsl:with-param name="referrer" select="$referrer"/>
  </xsl:apply-templates>
</xsl:template>

<xsl:template name="title.xref">
  <xsl:param name="target" select="."/>
  <xsl:choose>
    <xsl:when test="local-name($target) = 'figure'
                    or local-name($target) = 'example'
                    or local-name($target) = 'listing'
                    or local-name($target) = 'equation'
                    or local-name($target) = 'table'
                    or local-name($target) = 'dedication'
                    or local-name($target) = 'preface'
                    or local-name($target) = 'bibliography'
                    or local-name($target) = 'glossary'
                    or local-name($target) = 'index'
                    or local-name($target) = 'setindex'
                    or local-name($target) = 'colophon'">
      <xsl:call-template name="gentext.startquote"/>
      <xsl:apply-templates select="$target" mode="title.markup"/>
      <xsl:call-template name="gentext.endquote"/>
    </xsl:when>
    <xsl:otherwise>
      <fo:inline font-style="italic">
        <xsl:apply-templates select="$target" mode="title.markup"/>
      </fo:inline>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

<!-- From 'fo/docbook.xsl' -->

<!-- This is the original rule from 'docbook.xsl' which is applied to the
     DocBook root element. We use a copy here with a new mode attribute in
     order to apply it to the newly generated DocBook tree which contains the
     included listings.
-->
<xsl:template match="/" mode="originalRootRule">
  <xsl:call-template name="root.messages"/>

  <xsl:variable name="document.element" select="*[1]"/>

  <xsl:variable name="title">
    <xsl:choose>
      <xsl:when test="$document.element/title[1]">
        <xsl:value-of select="$document.element/title[1]"/>
      </xsl:when>
      <xsl:otherwise>[could not find document title]</xsl:otherwise>
    </xsl:choose>
  </xsl:variable>

  <fo:root font-family="{$body.font.family}"
           font-size="{$body.font.size}"
           text-align="{$alignment}"
           line-height="{$line-height}">
    <xsl:attribute name="language">
      <xsl:call-template name="l10n.language">
        <xsl:with-param name="target" select="/*[1]"/>
      </xsl:call-template>
    </xsl:attribute>

    <xsl:if test="$xep.extensions != 0">
      <xsl:call-template name="xep-document-information"/>
    </xsl:if>
    <xsl:call-template name="setup.pagemasters"/>
    <xsl:choose>
      <xsl:when test="$rootid != ''">
        <xsl:choose>
          <xsl:when test="count(key('id',$rootid)) = 0">
            <xsl:message terminate="yes">
              <xsl:text>ID '</xsl:text>
              <xsl:value-of select="$rootid"/>
              <xsl:text>' not found in document.</xsl:text>
            </xsl:message>
          </xsl:when>
          <xsl:otherwise>
            <xsl:if test="$fop.extensions != 0">
              <xsl:apply-templates select="key('id',$rootid)" mode="fop.outline"/>
            </xsl:if>
            <xsl:if test="$xep.extensions != 0">
              <xsl:variable name="bookmarks">
                <xsl:apply-templates select="key('id',$rootid)" mode="xep.outline"/>
              </xsl:variable>
              <xsl:if test="string($bookmarks) != ''">
                <rx:outline xmlns:rx="http://www.renderx.com/XSL/Extensions">
                  <xsl:copy-of select="$bookmarks"/>
                </rx:outline>
              </xsl:if>
            </xsl:if>
            <xsl:apply-templates select="key('id',$rootid)"/>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:when>
      <xsl:otherwise>
        <xsl:if test="$fop.extensions != 0">
          <xsl:apply-templates mode="fop.outline"/>
        </xsl:if>
        <xsl:if test="$xep.extensions != 0">
          <xsl:variable name="bookmarks">
            <xsl:apply-templates mode="xep.outline"/>
          </xsl:variable>
          <xsl:if test="string($bookmarks) != ''">
            <rx:outline xmlns:rx="http://www.renderx.com/XSL/Extensions">
              <xsl:copy-of select="$bookmarks"/>
            </rx:outline>
          </xsl:if>
        </xsl:if>
        <xsl:apply-templates/>
      </xsl:otherwise>
    </xsl:choose>

  </fo:root>
</xsl:template>


</xsl:stylesheet>  
