/**
 * A quick sort demonstration algorithm
 *
 * @author James Gosling
 * @author Kevin A. Smith
 * @version 1.3, 29 Feb 1996
 */
public class QSortAlgorithm {

  /** A generic version of C.A.R Hoare's Quick Sort algorithm. 
   * It handles sorted arrays, and arrays with duplicate keys.
   *
   * If you think of a one dimensional array as going from
   * the lowest index on the left to the highest index on the right
   * then the parameters to this function are lowest index or
   * left and highest index or right.  The first time you call
   * this function it will be with the parameters 0, a.length - 1.
   *
   * @param a       an integer array
   * @param lo0     left boundary of array partition
   * @param hi0     right boundary of array partition
   * @return        returns nothing, just for demonstration purpose
   */
  //< Include ID="QSMethod" label='The whole "QuickSort" method.'
  public static void QuickSort(int a[], int lo0, int hi0) {
    int lo = lo0;
    int hi = hi0;
    int mid;

    if ( hi0 > lo0) {
      // Arbitrarily establishing partition element as the midpoint of
      // the array.
      //
      mid = a[ ( lo0 + hi0 ) / 2 ];

      //< Include ID="whileLoop" label="Loop through the array until indices cross"
      while( lo <= hi ) {
        // find the first element that is greater than or equal to 
        // the partition element starting from the left Index.
        //
        while( ( lo < hi0 ) && ( a[lo] < mid ) ) {
          ++lo;
        }
        
        // find an element that is smaller than or equal to 
        // the partition element starting from the right Index.
        //
        while( ( hi > lo0 ) && ( a[hi] > mid ) ) --hi;
        
        // if the indexes have not crossed, swap
        if( lo <= hi ) {
          swap(a, lo, hi);    
          ++lo;
          --hi;
        }
      }
      //> Include
      
      //If the right index has not reached the left side of array
      //  must now sort the left partition.
      //
      if( lo0 < hi ) QuickSort( a, lo0, hi );
      
      // If the left index has not reached the right side of array
      // must now sort the right partition.
      //
      if( lo < hi0 ) QuickSort( a, lo, hi0 );
      
    }
  }
  //> Include
  
  public static void sort(int a[]) {
    QuickSort(a, 0, a.length - 1);
  }
  
  private static void swap(int a[], int i, int j) {
    int T;
    T = a[i]; 
    a[i] = a[j];
    a[j] = T;
  }
  
  private static void print(int a[]) {
    for(int i = 0; i < a.length; i++) {
      if (i > 0) System.out.print(", ");
      System.out.print(a[i]);
    }
    System.out.println();
  }
  
  public static void main(String argv[]) {
    int test[] = new int[] { 9, 5, 2, 6, 2, 7, 5, 1, 0, 4};
    print(test);
    sort(test);
    print(test);
  }
}
